import uuid

import pytest

import globus_sdk
from globus_sdk import GlobusApp, GlobusAppConfig, UserApp
from globus_sdk.testing import load_response
from globus_sdk.token_storage import MemoryTokenStorage


@pytest.fixture
def app() -> GlobusApp:
    config = GlobusAppConfig(token_storage=MemoryTokenStorage())
    return UserApp("test-app", client_id="client_id", config=config)


def test_client_inherits_environment_from_globus_app():
    config = GlobusAppConfig(token_storage=MemoryTokenStorage(), environment="sandbox")
    app = UserApp("test-app", client_id="client_id", config=config)

    client = globus_sdk.AuthClient(app=app)

    assert client.environment == "sandbox"


def test_client_environment_does_not_match_the_globus_app_environment():
    config = GlobusAppConfig(token_storage=MemoryTokenStorage(), environment="sandbox")
    app = UserApp("test-app", client_id="client_id", config=config)

    with pytest.raises(globus_sdk.GlobusSDKUsageError) as exc:
        globus_sdk.AuthClient(app=app, environment="preview")

    expected = "[Environment Mismatch] AuthClient's environment (preview) does not match the GlobusApp's configured environment (sandbox)."  # noqa: E501
    assert str(exc.value) == expected


def test_transfer_client_default_scopes(app):
    globus_sdk.TransferClient(app=app)

    assert [str(s) for s in app.scope_requirements["transfer.api.globus.org"]] == [
        "urn:globus:auth:scope:transfer.api.globus.org:all"
    ]


def test_transfer_client_add_app_data_access_scope(app):
    client = globus_sdk.TransferClient(app=app)

    collection_id = str(uuid.UUID(int=0))
    client.add_app_data_access_scope(collection_id)
    str_list = [str(s) for s in app.scope_requirements["transfer.api.globus.org"]]
    expected = f"urn:globus:auth:scope:transfer.api.globus.org:all[*https://auth.globus.org/scopes/{collection_id}/data_access]"  # noqa: E501
    assert expected in str_list


def test_timers_client_add_app_data_access_scope(app):
    client = globus_sdk.TimersClient(app=app)

    collection_id = str(uuid.UUID(int=0))
    client.add_app_transfer_data_access_scope(collection_id)
    str_list = [
        str(s) for s in app.scope_requirements[globus_sdk.TimersClient.resource_server]
    ]
    expected = f"{globus_sdk.TimersClient.scopes.timer}[urn:globus:auth:scope:transfer.api.globus.org:all[*https://auth.globus.org/scopes/{collection_id}/data_access]]"  # noqa: E501
    assert expected in str_list


def test_timers_client_add_app_flow_user_scope(app):
    client = globus_sdk.TimersClient(app=app)

    flow_id = str(uuid.UUID(int=0))
    client.add_app_flow_user_scope(flow_id)
    str_list = [
        str(s) for s in app.scope_requirements[globus_sdk.TimersClient.resource_server]
    ]

    flow_client = globus_sdk.SpecificFlowClient(flow_id, app=app)
    expected = f"{globus_sdk.TimersClient.scopes.timer}[{flow_client.scopes.user}]"  # noqa: E501
    assert expected in str_list


def test_specific_flow_client_add_app_data_access_scope(app):
    flow_id = str(uuid.UUID(int=1))
    client = globus_sdk.SpecificFlowClient(flow_id, app=app)

    collection_id = str(uuid.UUID(int=0))
    client.add_app_transfer_data_access_scope(collection_id)
    str_list = [str(s) for s in app.scope_requirements[client.resource_server]]
    expected = f"{client.scopes.user}[*urn:globus:auth:scope:transfer.api.globus.org:all[*https://auth.globus.org/scopes/{collection_id}/data_access]]"  # noqa: E501
    assert expected in str_list


def test_transfer_client_add_app_data_access_scope_chaining(app):
    collection_id_1 = str(uuid.UUID(int=1))
    collection_id_2 = str(uuid.UUID(int=2))
    (
        globus_sdk.TransferClient(app=app)
        .add_app_data_access_scope(collection_id_1)
        .add_app_data_access_scope(collection_id_2)
    )

    str_list = [str(s) for s in app.scope_requirements["transfer.api.globus.org"]]
    expected_1 = f"urn:globus:auth:scope:transfer.api.globus.org:all[*https://auth.globus.org/scopes/{collection_id_1}/data_access]"  # noqa: E501
    expected_2 = f"urn:globus:auth:scope:transfer.api.globus.org:all[*https://auth.globus.org/scopes/{collection_id_2}/data_access]"  # noqa: E501
    assert expected_1 in str_list
    assert expected_2 in str_list


def test_timers_client_add_app_flow_user_scope_chaining(app):
    flow_id_1 = str(uuid.UUID(int=1))
    flow_id_2 = str(uuid.UUID(int=2))
    (
        globus_sdk.TimersClient(app=app)
        .add_app_flow_user_scope(flow_id_1)
        .add_app_flow_user_scope(flow_id_2)
    )

    str_list = [
        str(s) for s in app.scope_requirements[globus_sdk.TimersClient.resource_server]
    ]
    flow_client_1 = globus_sdk.SpecificFlowClient(flow_id_1, app=app)
    expected_1 = f"{globus_sdk.TimersClient.scopes.timer}[{flow_client_1.scopes.user}]"  # noqa: E501
    flow_client_2 = globus_sdk.SpecificFlowClient(flow_id_2, app=app)
    expected_2 = f"{globus_sdk.TimersClient.scopes.timer}[{flow_client_2.scopes.user}]"  # noqa: E501

    assert expected_1 in str_list
    assert expected_2 in str_list


def test_transfer_client_add_app_data_access_scope_in_iterable(app):
    collection_id_1 = str(uuid.UUID(int=1))
    collection_id_2 = str(uuid.UUID(int=2))
    globus_sdk.TransferClient(app=app).add_app_data_access_scope(
        (collection_id_1, collection_id_2)
    )

    expected_1 = f"https://auth.globus.org/scopes/{collection_id_1}/data_access"
    expected_2 = f"https://auth.globus.org/scopes/{collection_id_2}/data_access"

    transfer_dependencies = []
    for scope in app.scope_requirements["transfer.api.globus.org"]:
        if scope.scope_string != str(globus_sdk.TransferClient.scopes.all):
            continue
        for dep in scope.dependencies:
            transfer_dependencies.append((dep.scope_string, dep.optional))

    assert (expected_1, True) in transfer_dependencies
    assert (expected_2, True) in transfer_dependencies


def test_timers_client_add_app_data_access_scope_in_iterable(app):
    collection_id_1 = str(uuid.UUID(int=1))
    collection_id_2 = str(uuid.UUID(int=2))
    globus_sdk.TimersClient(app=app).add_app_transfer_data_access_scope(
        (collection_id_1, collection_id_2)
    )

    expected_1 = f"https://auth.globus.org/scopes/{collection_id_1}/data_access"
    expected_2 = f"https://auth.globus.org/scopes/{collection_id_2}/data_access"

    transfer_dependencies = []
    for scope in app.scope_requirements[globus_sdk.TimersClient.resource_server]:
        if scope.scope_string != str(globus_sdk.TimersClient.scopes.timer):
            continue
        for dep in scope.dependencies:
            if dep.scope_string != str(globus_sdk.TransferClient.scopes.all):
                continue
            for subdep in dep.dependencies:
                transfer_dependencies.append((subdep.scope_string, subdep.optional))

    assert (expected_1, True) in transfer_dependencies
    assert (expected_2, True) in transfer_dependencies


def test_timers_client_add_app_flow_user_scope_in_iterable(app):
    flow_id_1 = str(uuid.UUID(int=1))
    flow_id_2 = str(uuid.UUID(int=2))
    globus_sdk.TimersClient(app=app).add_app_flow_user_scope((flow_id_1, flow_id_2))

    timer_dependencies = [
        scope_dep.scope_string
        for scope in app.scope_requirements[globus_sdk.TimersClient.resource_server]
        for scope_dep in scope.dependencies
    ]
    flow_client_1 = globus_sdk.SpecificFlowClient(flow_id_1, app=app)
    flow_client_2 = globus_sdk.SpecificFlowClient(flow_id_2, app=app)

    assert flow_client_1.scopes.user.scope_string in timer_dependencies
    assert flow_client_2.scopes.user.scope_string in timer_dependencies


def test_transfer_client_add_app_data_access_scope_catches_bad_uuid(app):
    with pytest.raises(ValueError, match="'collection_ids' must be a valid UUID"):
        globus_sdk.TransferClient(app=app).add_app_data_access_scope("foo")


def test_transfer_client_add_app_data_access_scope_catches_bad_uuid_in_iterable(app):
    collection_id_1 = str(uuid.UUID(int=1))
    with pytest.raises(ValueError, match=r"'collection_ids\[1\]' must be a valid UUID"):
        globus_sdk.TransferClient(app=app).add_app_data_access_scope(
            [collection_id_1, "foo"]
        )


def test_timers_client_add_app_data_access_scope_catches_bad_uuid(app):
    with pytest.raises(ValueError, match="'collection_ids' must be a valid UUID"):
        globus_sdk.TimersClient(app=app).add_app_transfer_data_access_scope("foo")


def test_timers_client_add_app_flow_user_scope_catches_bad_uuid(app):
    with pytest.raises(ValueError, match="'flow_ids' must be a valid UUID"):
        globus_sdk.TimersClient(app=app).add_app_flow_user_scope("foo")


def test_timers_client_add_app_data_access_scope_catches_bad_uuid_in_iterable(app):
    collection_id_1 = str(uuid.UUID(int=1))
    with pytest.raises(ValueError, match=r"'collection_ids\[1\]' must be a valid UUID"):
        globus_sdk.TimersClient(app=app).add_app_transfer_data_access_scope(
            [collection_id_1, "foo"]
        )


def test_timers_client_add_app_flow_user_scope_catches_bad_uuid_in_iterable(app):
    flow_id = str(uuid.UUID(int=1))
    with pytest.raises(ValueError, match=r"'flow_ids\[1\]' must be a valid UUID"):
        globus_sdk.TimersClient(app=app).add_app_flow_user_scope([flow_id, "foo"])


def test_auth_client_default_scopes(app):
    globus_sdk.AuthClient(app=app)

    str_list = [str(s) for s in app.scope_requirements["auth.globus.org"]]
    assert "openid" in str_list
    assert "profile" in str_list
    assert "email" in str_list


def test_groups_client_default_scopes(app):
    globus_sdk.GroupsClient(app=app)

    assert [str(s) for s in app.scope_requirements["groups.api.globus.org"]] == [
        "urn:globus:auth:scope:groups.api.globus.org:view_my_groups_and_memberships"
    ]


def test_search_client_default_scopes(app):
    globus_sdk.SearchClient(app=app)

    assert [str(s) for s in app.scope_requirements["search.api.globus.org"]] == [
        "urn:globus:auth:scope:search.api.globus.org:search"
    ]


def test_timer_client_default_scopes(app):
    globus_sdk.TimersClient(app=app)

    timer_client_id = "524230d7-ea86-4a52-8312-86065a9e0417"
    str_list = [str(s) for s in app.scope_requirements[timer_client_id]]
    assert str_list == [f"https://auth.globus.org/scopes/{timer_client_id}/timer"]


def test_flows_client_default_scopes(app):
    globus_sdk.FlowsClient(app=app)

    flows_client_id = "eec9b274-0c81-4334-bdc2-54e90e689b9a"
    str_list = [str(s) for s in app.scope_requirements["flows.globus.org"]]
    assert len(str_list) == 1
    assert str_list == [f"https://auth.globus.org/scopes/{flows_client_id}/all"]


def test_specific_flow_client_default_scopes(app):
    globus_sdk.SpecificFlowClient("flow_id", app=app)

    assert [str(s) for s in app.scope_requirements["flow_id"]] == [
        "https://auth.globus.org/scopes/flow_id/flow_flow_id_user"
    ]


def test_gcs_client_default_scopes(app):
    meta = load_response(globus_sdk.GCSClient.get_gcs_info).metadata
    endpoint_client_id = meta["endpoint_client_id"]
    domain_name = meta["domain_name"]

    globus_sdk.GCSClient(domain_name, app=app)

    assert [str(s) for s in app.scope_requirements[endpoint_client_id]] == [
        f"urn:globus:auth:scope:{endpoint_client_id}:manage_collections"
    ]



# test that the internal guards module provides valid and well-formed type-guards
import typing as t

from globus_sdk._internal import guards


def get_any() -> t.Any:
    return 1


x = get_any()
t.assert_type(x, t.Any)

# test is_list_of
if guards.is_list_of(x, str):
    t.assert_type(x, list[str])
elif guards.is_list_of(x, int):
    t.assert_type(x, list[int])

# test is_optional
if guards.is_optional(x, float):
    t.assert_type(x, float | None)
elif guards.is_optional(x, bytes):
    t.assert_type(x, bytes | None)


# test is_optional_list_of
if guards.is_optional_list_of(x, type(None)):
    t.assert_type(x, list[None] | None)
elif guards.is_optional_list_of(x, dict):
    t.assert_type(x, list[dict[t.Any, t.Any]] | None)



from globus_sdk.scopes._graph_parser import ScopeGraph


def test_graph_str_single_node():
    g = ScopeGraph.parse("foo")
    clean_str = _blank_lines_removed(str(g))
    assert clean_str == """\
digraph scopes {
  rankdir="LR";
  foo
}"""


def test_graph_str_single_optional_node():
    g = ScopeGraph.parse("*foo")
    clean_str = _blank_lines_removed(str(g))
    assert clean_str == """\
digraph scopes {
  rankdir="LR";
  *foo
}"""


def test_graph_str_single_dependency():
    g = ScopeGraph.parse("foo[bar]")
    clean_str = _blank_lines_removed(str(g))
    assert clean_str == """\
digraph scopes {
  rankdir="LR";
  foo
  foo -> bar;
}"""


def test_graph_str_optional_dependency():
    g = ScopeGraph.parse("foo[bar[*baz]]")
    clean_str = _blank_lines_removed(str(g))
    assert clean_str == """\
digraph scopes {
  rankdir="LR";
  foo
  foo -> bar;
  bar -> baz [ label = "optional" ];
}"""


def _blank_lines_removed(s: str) -> str:
    return "\n".join(line for line in s.split("\n") if line.strip() != "")



from globus_sdk import IterableResponse
from globus_sdk.scopes.consents import ConsentForest


class GetConsentsResponse(IterableResponse):
    """
    Response class specific to the Get Consents API

    Provides iteration on the "consents" array in the response.
    """

    default_iter_key = "consents"

    def to_forest(self) -> ConsentForest:
        """
        Creates a ConsentForest from the consents in this response.

        ConsentForest is a convenience class to make interacting with the
            tree of consents simpler.
        """
        return ConsentForest(self)



import uuid

from globus_sdk.testing.models import RegisteredResponse, ResponseSet

TUNNEL_ID = str(uuid.uuid4())

GET_TUNNEL_EVENTS_DOC = {
    "data": [
        {
            "type": "TunnelEvent",
            "id": 2768,
            "attributes": {
                "code": "STARTED",
                "is_error": False,
                "description": "started",
                "details": "Attempting tunnel establishment",
                "time": "2026-02-12T21:59:01.857473",
            },
        },
        {
            "type": "TunnelEvent",
            "id": 2769,
            "attributes": {
                "code": "TUNNEL_ACTIVE",
                "is_error": False,
                "description": "tunnel is active",
                "details": "Tunnel has been established",
                "time": "2026-02-12T21:59:02.876253",
            },
        },
        {
            "type": "TunnelEvent",
            "id": 2777,
            "attributes": {
                "code": "TUNNEL_STOPPED",
                "is_error": False,
                "description": "tunnel has been stopped",
                "details": "Tunnel stopped as requested.",
                "time": "2026-02-12T22:12:03.655877",
            },
        },
    ],
    "links": None,
    "meta": {"request_id": "655TZe5vm"},
}


RESPONSES = ResponseSet(
    metadata={
        "tunnel_id": TUNNEL_ID,
    },
    default=RegisteredResponse(
        service="transfer",
        path=f"/v2/tunnels/{TUNNEL_ID}/events",
        json=GET_TUNNEL_EVENTS_DOC,
        method="GET",
    ),
)



import globus_sdk
from globus_sdk.globus_app import ClientApp

# Confidential Client ID/Secret - <replace these with real client values>
CONFIDENTIAL_CLIENT_ID = "..."
CONFIDENTIAL_CLIENT_SECRET = "..."

# Globus Tutorial Collection 1
# https://app.globus.org/file-manager/collections/6c54cade-bde5-45c1-bdea-f4bd71dba2cc
ENDPOINT_HOSTNAME = "https://b7a4f1.75bc.data.globus.org"
STORAGE_GATEWAY_ID = "947460f6-3fcd-4acc-9683-d71e14e5ace1"
MAPPED_COLLECTION_ID = "6c54cade-bde5-45c1-bdea-f4bd71dba2cc"


def main():
    with ClientApp(
        "my-simple-client-collection",
        client_id=CONFIDENTIAL_CLIENT_ID,
        client_secret=CONFIDENTIAL_CLIENT_SECRET,
    ) as app:
        with globus_sdk.GCSClient(ENDPOINT_HOSTNAME, app=app) as gcs_client:
            create_guest_collection(gcs_client)


def create_guest_collection(gcs_client: globus_sdk.GCSClient):
    # Comment out this line if the mapped collection is high assurance
    attach_data_access_scope(gcs_client, MAPPED_COLLECTION_ID)

    ensure_user_credential(gcs_client)

    collection_request = globus_sdk.GuestCollectionDocument(
        public=True,
        collection_base_path="/",
        display_name="example_guest_collection",
        mapped_collection_id=MAPPED_COLLECTION_ID,
    )

    collection = gcs_client.create_collection(collection_request)
    print(f"Created guest collection. Collection ID: {collection['id']}")


def attach_data_access_scope(gcs_client, collection_id):
    """Compose and attach a ``data_access`` scope for the supplied collection"""
    endpoint_scopes = gcs_client.get_gcs_endpoint_scopes(gcs_client.endpoint_client_id)
    collection_scopes = gcs_client.get_gcs_collection_scopes(collection_id)

    data_access = globus_sdk.Scope(collection_scopes.data_access, optional=True)
    manage_collections = globus_sdk.Scope(
        endpoint_scopes.manage_collections, dependencies=(data_access,)
    )

    gcs_client.add_app_scope(manage_collections)


def ensure_user_credential(gcs_client):
    """
    Ensure that the client has a user credential on the client.
    This is the mapping between Globus Auth (OAuth2) and the local system's permissions.
    """
    # Depending on the endpoint & storage gateway, this request document may need to
    # include more complex information such as a local username.
    # Consult with the endpoint owner for more detailed info on user mappings and
    # other specific requirements.
    req = globus_sdk.UserCredentialDocument(storage_gateway_id=STORAGE_GATEWAY_ID)
    try:
        gcs_client.create_user_credential(req)
    except globus_sdk.GCSAPIError as err:
        # A user credential already exists, no need to create it.
        if err.http_status != 409:
            raise


if __name__ == "__main__":
    main()



from .auth import AuthScopes
from .compute import ComputeScopes
from .flows import FlowsScopes, SpecificFlowScopes
from .gcs import GCSCollectionScopes, GCSEndpointScopes
from .groups import GroupsScopes, NexusScopes
from .search import SearchScopes
from .timers import TimersScopes
from .transfer import TransferScopes

__all__ = (
    "AuthScopes",
    "ComputeScopes",
    "FlowsScopes",
    "SpecificFlowScopes",
    "GCSEndpointScopes",
    "GCSCollectionScopes",
    "GroupsScopes",
    "NexusScopes",
    "SearchScopes",
    "TimersScopes",
    "TransferScopes",
)



import uuid

import pytest

from globus_sdk.exc import ErrorSubdocument
from globus_sdk.gare import (
    GARE,
    GlobusAuthorizationParameters,
    _variants,
    has_gares,
    is_gare,
    to_gare,
    to_gares,
)
from globus_sdk.testing import construct_error


@pytest.mark.parametrize(
    "error_dict, status, expected_required_scopes, expected_message",
    (
        (
            {
                "code": "ConsentRequired",
                "message": "Missing required foo_bar consent",
                "request_id": "WmMV97A1w",
                "required_scopes": [
                    "urn:globus:auth:scope:transfer.api.globus.org:all[*foo *bar]"
                ],
                "resource": "/transfer",
            },
            403,
            ["urn:globus:auth:scope:transfer.api.globus.org:all[*foo *bar]"],
            "Missing required foo_bar consent",
        ),
        (
            {
                "code": "ConsentRequired",
                "required_scope": (
                    "urn:globus:auth:scope:transfer.api.globus.org:all[*foo *bar]"
                ),
                "description": "Missing required foo_bar consent",
            },
            401,
            ["urn:globus:auth:scope:transfer.api.globus.org:all[*foo *bar]"],
            "Missing required foo_bar consent",
        ),
        pytest.param(
            {
                "error": "dependent_consent_required",
                "error_description": "User must approve your client using scopes. See 'unapproved_scopes'.",  # noqa: E501
                "errors": [
                    {
                        "code": "DEPENDENT_CONSENT_REQUIRED",
                        "id": str(uuid.uuid4()),
                        "title": "User must approve your client using scopes. See 'unapproved_scopes'.",  # noqa: E501
                        "unapproved_scopes": [
                            "https://auth.globus.org/scopes/00000000-ec3c-427d-bfb5-51049530122b/flow_00000000_ec3c_427d_bfb5_51049530122b_user"  # noqa: E501
                        ],
                        "status": "403",
                        "detail": "User hasn't approved the following scopes: https://auth.globus.org/scopes/00000000-ec3c-427d-bfb5-51049530122b/flow_00000000_ec3c_427d_bfb5_51049530122b_user",  # noqa: E501
                    }
                ],
            },
            403,
            [
                "https://auth.globus.org/scopes/00000000-ec3c-427d-bfb5-51049530122b/flow_00000000_ec3c_427d_bfb5_51049530122b_user"  # noqa: E501
            ],
            None,
            id="Auth 'dependent_consent_required' error",
        ),
    ),
)
def test_create_auth_requirements_error_from_consent_error(
    error_dict, status, expected_required_scopes, expected_message
):
    """
    Test that various ConsentRequired error shapes can be detected and converted
    to a GlobusAuthRequirementsError.
    """
    # Create various supplementary objects representing this error
    error_subdoc = ErrorSubdocument(error_dict)
    api_error = construct_error(body=error_dict, http_status=status)

    for error in (error_dict, error_subdoc, api_error):
        # Test boolean utility functions
        assert is_gare(error)
        assert has_gares([error])

        # Check that this only produces one error
        assert len(to_gares([error])) == 1

        # Create a Globus Auth requirements error from the original error
        authreq_error = to_gare(error)
        assert isinstance(authreq_error, GARE)
        assert authreq_error.code == "ConsentRequired"
        assert (
            authreq_error.authorization_parameters.required_scopes
            == expected_required_scopes
        )
        if expected_message is not None:
            assert (
                authreq_error.authorization_parameters.session_message
                == expected_message
            )


@pytest.mark.parametrize(
    "authorization_parameters",
    (
        {
            "session_message": (
                "To gain access you need to authenticate with your baz identity"
            ),
            "session_required_identities": ["urn:globus:auth:identity:baz"],
            "session_required_mfa": True,
        },
        {
            "session_message": (
                "You need to authenticate with an identity that "
                "matches the required policies"
            ),
            "session_required_policies": ["foo", "baz"],
        },
        {
            "session_message": (
                "You need to authenticate with an identity that "
                "belongs to an authorized domain"
            ),
            "session_required_single_domain": ["foo.com", "baz.org"],
        },
        {
            "session_message": "You need to re-authenticate",
            "session_required_single_domain": ["foo.com", "baz.org"],
            "prompt": "login",
        },
    ),
)
def test_create_auth_requirements_error_from_authorization_error(
    authorization_parameters,
):
    """
    Test that various authorization parameters error shapes can be detected and
    converted to a GlobusAuthRequirementsError.
    """
    # Create various supplementary objects representing this error
    error_dict = {"authorization_parameters": authorization_parameters}
    error_subdoc = ErrorSubdocument(error_dict)
    api_error = construct_error(body=error_dict, http_status=403)

    for error in (error_dict, error_subdoc, api_error):
        # Test boolean utility functions
        assert is_gare(error)
        assert has_gares([error])

        # Check that this only produces one error
        assert len(to_gares([error])) == 1

        # Create a Globus Auth requirements error from a legacy
        # authorization parameters format error
        authreq_error = to_gare(error)
        assert isinstance(authreq_error, GARE)

        # Check that the default error code is set
        assert authreq_error.code == "AuthorizationRequired"

        # Iterate over the expected attributes and check that they match
        for name, value in authorization_parameters.items():
            assert getattr(authreq_error.authorization_parameters, name) == value


@pytest.mark.parametrize(
    "authorization_parameters",
    (
        {
            "session_message": (
                "You need to authenticate with an identity that "
                "matches the required policies"
            ),
            "session_required_policies": ["foo", "baz"],
        },
        {
            "session_message": (
                "You need to authenticate with an identity that "
                "belongs to an authorized domain"
            ),
            "session_required_single_domain": ["foo.com", "baz.org"],
        },
    ),
)
def test_create_auth_requirements_error_from_authorization_error_csv(
    authorization_parameters,
):
    """
    Test that authorization parameters error shapes that provide lists as comma-
    delimited values can be detected and converted to a GlobusAuthRequirementsError
    normalizing to lists of strings for those values.
    """
    # Create various supplementary objects representing this error
    error_dict = {"authorization_parameters": {}}
    for key, value in authorization_parameters.items():
        if key in ("session_required_policies", "session_required_single_domain"):
            # Convert the list to a comma-separated string for known variants
            error_dict["authorization_parameters"][key] = ",".join(value)
        else:
            error_dict["authorization_parameters"][key] = value

    error_subdoc = ErrorSubdocument(error_dict)
    api_error = construct_error(body=error_dict, http_status=403)

    for error in (error_dict, error_subdoc, api_error):
        # Test boolean utility functions
        assert is_gare(error)
        assert has_gares([error])

        # Check that this only produces one error
        assert len(to_gares([error])) == 1

        # Create a Globus Auth requirements error from a legacy
        # authorization parameters format error
        authreq_error = to_gare(error)
        assert isinstance(authreq_error, GARE)

        # Check that the default error code is set
        assert authreq_error.code == "AuthorizationRequired"

        # Iterate over the expected attributes and check that they match
        for name, value in authorization_parameters.items():
            assert getattr(authreq_error.authorization_parameters, name) == value


def test_create_auth_requirements_errors_from_multiple_errors():
    """
    Test that a GlobusAPIError with multiple subdocuments is converted to multiple
    GlobusAuthRequirementsErrors, and additionally test that this is correct even
    when mingled with other accepted data types.
    """
    consent_errors = construct_error(
        body={
            "errors": [
                {
                    "code": "ConsentRequired",
                    "message": "Missing required foo_bar consent",
                    "authorization_parameters": {
                        "required_scopes": [
                            "urn:globus:auth:scope:transfer.api.globus.org:all[*bar]"
                        ],
                        "session_message": "Missing required foo_bar consent",
                    },
                },
                {
                    "code": "ConsentRequired",
                    "message": "Missing required foo_baz consent",
                    "authorization_parameters": {
                        "required_scopes": [
                            "urn:globus:auth:scope:transfer.api.globus.org:all[*baz]"
                        ],
                        "session_message": "Missing required foo_baz consent",
                    },
                },
            ]
        },
        http_status=403,
    )

    authorization_error = construct_error(
        body={
            "authorization_parameters": {
                "session_message": (
                    "You need to authenticate with an identity that "
                    "matches the required policies"
                ),
                "session_required_policies": ["foo", "baz"],
            }
        },
        http_status=403,
    )

    not_an_error = construct_error(
        body={
            "code": "NotAnError",
            "message": "This is not an error",
        },
        http_status=403,
    )

    all_errors = [consent_errors, not_an_error, authorization_error]

    # Test boolean utility function
    assert has_gares(all_errors)

    # Create auth requirements errors from a all errors
    authreq_errors = to_gares(all_errors)
    assert isinstance(authreq_errors, list)
    assert len(authreq_errors) == 3

    # Check that errors properly converted
    for authreq_error in authreq_errors:
        assert isinstance(authreq_error, GARE)

    # Check that the proper auth requirements errors were produced
    assert authreq_errors[0].code == "ConsentRequired"
    assert authreq_errors[0].authorization_parameters.required_scopes == [
        "urn:globus:auth:scope:transfer.api.globus.org:all[*bar]"
    ]
    assert (
        authreq_errors[0].authorization_parameters.session_message
        == "Missing required foo_bar consent"
    )
    assert authreq_errors[1].code == "ConsentRequired"
    assert authreq_errors[1].authorization_parameters.required_scopes == [
        "urn:globus:auth:scope:transfer.api.globus.org:all[*baz]"
    ]
    assert (
        authreq_errors[1].authorization_parameters.session_message
        == "Missing required foo_baz consent"
    )
    assert authreq_errors[2].code == "AuthorizationRequired"
    assert authreq_errors[2].authorization_parameters.session_required_policies == [
        "foo",
        "baz",
    ]
    assert authreq_errors[2].authorization_parameters.session_message == (
        "You need to authenticate with an identity that matches the required policies"
    )


def test_create_auth_requirements_error_from_legacy_authorization_error_with_code():
    """
    Test that legacy authorization parameters error shapes that provide a `code` can be
    detected and converted to a GlobusAuthRequirementsError while retaining the `code`.
    """
    # Create a legacy authorization parameters error with a code
    error_dict = {
        "code": "UnsatisfiedPolicy",
        "authorization_parameters": {
            "session_message": (
                "You need to authenticate with an identity that "
                "matches the required policies"
            ),
            "session_required_policies": "foo,baz",
        },
    }

    # Create various supplementary objects representing this error
    error_subdoc = ErrorSubdocument(error_dict)
    api_error = construct_error(body=error_dict, http_status=403)

    for error in (error_dict, error_subdoc, api_error):
        # Test boolean utility functions
        assert is_gare(error)
        assert has_gares([error])

        # Check that this only produces one error
        assert len(to_gares([error])) == 1

        # Create a Globus Auth requirements error from a legacy
        # authorization parameters format error
        authreq_error = to_gare(error)
        assert isinstance(authreq_error, GARE)

        # Check that the custom error code is set
        assert authreq_error.code == "UnsatisfiedPolicy"

        # Iterate over the expected attributes and check that they match
        assert authreq_error.authorization_parameters.session_required_policies == [
            "foo",
            "baz",
        ]


def test_backward_compatibility_consent_required_error():
    """
    Test that a consent required error with a comingled backward-compatible
    data schema is converted to a GlobusAuthRequirementsError.
    """
    # Create an API error with a backward compatible data schema using
    # distinct values for duplicative fields to facilitate testing
    # (in practice these would be the same)
    error = construct_error(
        body={
            "code": "ConsentRequired",
            "message": "Missing required foo_bar consent",
            "request_id": "WmMV97A1w",
            "required_scopes": [
                "urn:globus:auth:scope:transfer.api.globus.org:all[*foo *bar]"
            ],
            "resource": "/transfer",
            "authorization_parameters": {
                "session_message": "Missing baz consent",
                "required_scopes": [
                    "urn:globus:auth:scope:transfer.api.globus.org:all[*baz]"
                ],
                "optional": "A non-canonical field",
            },
        },
        http_status=403,
    )

    # Test boolean utility functions
    assert is_gare(error)
    assert has_gares([error])

    # Check that this only produces one error
    assert len(to_gares([error])) == 1

    # Create a Globus Auth requirements error
    authreq_error = to_gare(error)
    assert isinstance(authreq_error, GARE)
    assert authreq_error.code == "ConsentRequired"
    assert authreq_error.authorization_parameters.required_scopes == [
        "urn:globus:auth:scope:transfer.api.globus.org:all[*baz]"
    ]
    assert (
        authreq_error.authorization_parameters.session_message == "Missing baz consent"
    )

    # Test that only suppotred fields are present in the dict
    assert authreq_error.to_dict() == {
        "code": "ConsentRequired",
        "authorization_parameters": {
            "session_message": "Missing baz consent",
            "required_scopes": [
                "urn:globus:auth:scope:transfer.api.globus.org:all[*baz]"
            ],
        },
    }

    # Test that extra fields are present in the dict
    assert authreq_error.to_dict(include_extra=True) == {
        "code": "ConsentRequired",
        "message": "Missing required foo_bar consent",
        "request_id": "WmMV97A1w",
        "required_scopes": [
            "urn:globus:auth:scope:transfer.api.globus.org:all[*foo *bar]"
        ],
        "resource": "/transfer",
        "authorization_parameters": {
            "session_message": "Missing baz consent",
            "required_scopes": [
                "urn:globus:auth:scope:transfer.api.globus.org:all[*baz]"
            ],
            "optional": "A non-canonical field",
        },
    }


@pytest.mark.parametrize(
    "target_class, data, expect_message",
    [
        (  # missing 'code'
            GARE,
            {"authorization_parameters": {"session_required_policies": "foo"}},
            "'code' must be a string",
        ),
        (  # missing 'authorization_parameters'
            _variants.LegacyAuthorizationParametersError,
            {},
            (
                "'authorization_parameters' must be a 'LegacyAuthorizationParameters' "
                "object or a dictionary"
            ),
        ),
        (  # missing 'code'
            _variants.LegacyConsentRequiredTransferError,
            {"required_scopes": []},
            "'code' must be the string 'ConsentRequired'",
        ),
        (  # missing 'code'
            _variants.LegacyConsentRequiredAPError,
            {"required_scope": "foo"},
            "'code' must be the string 'ConsentRequired'",
        ),
    ],
)
def test_error_from_dict_insufficient_input(target_class, data, expect_message):
    """ """
    with pytest.raises(ValueError) as exc_info:
        target_class.from_dict(data)

    assert str(exc_info.value) == expect_message


@pytest.mark.parametrize(
    "target_class",
    [
        GlobusAuthorizationParameters,
        _variants.LegacyAuthorizationParameters,
    ],
)
def test_authorization_parameters_from_empty_dict(target_class):
    """ """
    authorization_params = target_class.from_dict({})
    assert authorization_params.to_dict() == {}


def test_gare_repr_shows_attrs():
    error_doc = GARE(
        code="NeedsReauth",
        authorization_parameters={"session_required_policies": ["foo"]},
    )

    # the repr will include the parameters repr -- tested separately below
    assert repr(error_doc) == (
        "GARE("
        "code='NeedsReauth', "
        f"authorization_parameters={error_doc.authorization_parameters!r}"
        ")"
    )


def test_gare_repr_indicates_presence_of_extra():
    error_doc_no_extra = GARE(
        code="NeedsReauth",
        authorization_parameters={"session_required_policies": ["foo"]},
    )

    error_doc_with_extra = GARE(
        code="NeedsReauth",
        authorization_parameters={"session_required_policies": ["foo"]},
        extra={"alpha": "beta"},
    )

    assert "extra=..." not in repr(error_doc_no_extra)
    assert "extra=..." in repr(error_doc_with_extra)


def test_authorization_parameters_repr_shows_all_attrs():
    params = GlobusAuthorizationParameters()
    assert repr(params) == (
        "GlobusAuthorizationParameters("
        "session_message=None, "
        "session_required_identities=None, "
        "session_required_policies=None, "
        "session_required_single_domain=None, "
        "session_required_mfa=None, "
        "required_scopes=None, "
        "prompt=None"
        ")"
    )


def test_authorization_parameters_repr_indicates_presence_of_extra():
    params_no_extra = GlobusAuthorizationParameters()
    params_with_extra = GlobusAuthorizationParameters(extra={"gamma": "delta"})

    assert "extra=..." not in repr(params_no_extra)
    assert "extra=..." in repr(params_with_extra)


@pytest.mark.parametrize("method", (to_gare, to_gares))
def test_create_gare_from_policy_error_when_non_gare_subdocuments_are_present(
    method,
):
    # this error data is based on a real API error shape from Auth
    # the top-level error is a GARE; but the subdocuments are not
    policy_id = str(uuid.uuid1())
    error_dict = {
        "errors": [
            {
                "detail": (
                    "To access this project you must have an identity with admin "
                    "privileges in session within the last 30 minutes."
                ),
                "id": "4a156297-a2e5-4095-a13c-ba9486035f79",
                "title": "Forbidden",
                "status": "403",
                "code": "FORBIDDEN",
            }
        ],
        "error": "forbidden",
        "error_description": "Forbidden",
        "authorization_parameters": {
            "session_required_policies": [policy_id],
            "session_message": (
                "To access this project you must have an identity with admin "
                "privileges in session within the last 30 minutes."
            ),
        },
    }
    api_error = construct_error(body=error_dict, http_status=403)

    # pass singular or plural, to match the relevant method
    if method is to_gare:
        gare = method(api_error)
    else:
        all_gares = method([api_error])
        assert len(all_gares) == 1
        gare = all_gares[0]

    assert isinstance(gare, GARE)
    # no 'code' was provided in the original error data, so the default will be induced
    assert gare.code == "AuthorizationRequired"
    # there are no scopes
    assert gare.authorization_parameters.required_scopes is None
    # the message matches the input doc
    assert (
        gare.authorization_parameters.session_message
        == error_dict["authorization_parameters"]["session_message"]
    )
    # and the policy ID is provided in the required policies field
    assert gare.authorization_parameters.session_required_policies == [policy_id]



import uuid

from globus_sdk.testing.models import RegisteredResponse, ResponseSet

_bookmarks = [
    {
        "bookmark_id": str(uuid.uuid4()),
        "collection_id": str(uuid.uuid4()),
        "name": "public datasets",
        "path": "/data_repository/public",
        "pinned": True,
    },
    {
        "bookmark_id": str(uuid.uuid4()),
        "collection_id": str(uuid.uuid4()),
        "name": "private datasets",
        "path": "/data_repository/private",
        "pinned": False,
    },
]

RESPONSES = ResponseSet(
    default=RegisteredResponse(
        service="transfer",
        method="GET",
        path="/v2/bookmarks",
        json={
            "data": [
                {
                    "type": "Bookmark",
                    "id": b["bookmark_id"],
                    "attributes": {
                        "name": b["name"],
                        "path": b["path"],
                        "pinned": b["pinned"],
                    },
                    "relationships": {
                        "collection": {
                            "data": {
                                "type": "Collection",
                                "id": b["collection_id"],
                            }
                        }
                    },
                }
                for b in _bookmarks
            ],
            "meta": {
                "request_id": "xj8AWyQr8",
            },
            "included": [
                {
                    "type": "Collection",
                    "id": b["collection_id"],
                    "attributes": {
                        "display_name": "Wotsomatta U Dataset Collection",
                        "high_assurance": False,
                    },
                }
                for b in _bookmarks
            ],
        },
    ),
    metadata={"bookmarks": _bookmarks},
)



"""
Import the modules from the lazy import table in "all" possible orders. This ensures
that there cannot be any user-triggerable import cycles.

Note that testing all permutations is infeasibly expensive.

This is kept in the non-pytest dir because it may be written in pytest but it is not
part of the normal testsuite.
"""

import itertools
import os
import subprocess
import sys

import pytest

import globus_sdk
from globus_sdk._internal.lazy_import import find_source_module

PYTHON_BINARY = os.environ.get("GLOBUS_TEST_PY", sys.executable)

MODULE_NAMES = sorted(
    {
        find_source_module("globus_sdk", "__init__.pyi", attr).lstrip(".")
        for attr in globus_sdk.__all__
        if not attr.startswith("_")
    }
)


@pytest.mark.parametrize(
    "first_module, second_module", itertools.permutations(MODULE_NAMES, 2)
)
def test_import_pairwise(first_module, second_module):
    command = (
        f"from globus_sdk.{first_module} import *; "
        f"from globus_sdk.{second_module} import *"
    )
    proc = subprocess.Popen(
        f'{PYTHON_BINARY} -c "{command}"',
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    status = proc.wait()
    assert status == 0, str(proc.communicate())
    proc.stdout.close()
    proc.stderr.close()


@pytest.mark.parametrize("first_module", MODULE_NAMES)
def test_import_all_each_first(first_module):
    command = f"from globus_sdk.{first_module} import *; " + "; ".join(
        f"from globus_sdk.{mod} import *" for mod in MODULE_NAMES if mod != first_module
    )
    proc = subprocess.Popen(
        f'{PYTHON_BINARY} -c "{command}"',
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    status = proc.wait()
    assert status == 0, str(proc.communicate())
    proc.stdout.close()
    proc.stderr.close()



import os
import pathlib
from unittest import mock

import pytest

import globus_sdk.config


def test_get_service_url():
    """
    Confirms get_service_url returns expected results
    Tests environments, services, and missing values
    """
    assert (
        globus_sdk.config.get_service_url("auth", environment="production")
        == "https://auth.globus.org/"
    )
    assert (
        globus_sdk.config.get_service_url("transfer", environment="production")
        == "https://transfer.api.globus.org/"
    )
    assert (
        globus_sdk.config.get_service_url("auth", environment="preview")
        == "https://auth.preview.globus.org/"
    )
    assert (
        globus_sdk.config.get_service_url("search", environment="preview")
        == "https://search.api.preview.globus.org/"
    )
    assert (
        globus_sdk.config.get_service_url("timer", environment="preview")
        == "https://preview.timer.automate.globus.org/"
    )

    with pytest.raises(ValueError):
        globus_sdk.config.get_service_url("auth", environment="nonexistent")


ca_bundle_file = pathlib.Path(__file__).parent.absolute() / "CA-Bundle.cert"
ca_bundle_directory = pathlib.Path(__file__).parent.absolute()
ca_bundle_non_existent = pathlib.Path(__file__).parent.absolute() / "bogus.bogus"


@pytest.mark.parametrize(
    "value, expected_result",
    [(x, True) for x in ["1", "YES", "true", "t", "True", "ON"]]
    + [(x, False) for x in ["0", "NO", "false", "f", "False", "OFF"]]
    + [(str(ca_bundle_file), str(ca_bundle_file))]
    + [
        ("invalid", ValueError),
        ("1.0", ValueError),
        ("0.0", ValueError),
        (str(ca_bundle_directory), ValueError),
        (str(ca_bundle_non_existent), ValueError),
    ],
)
def test_get_ssl_verify(value, expected_result, monkeypatch):
    """
    Confirms bool cast returns correct bools from sets of string values
    """
    monkeypatch.setenv("GLOBUS_SDK_VERIFY_SSL", value)
    if expected_result is not ValueError:
        assert globus_sdk.config.get_ssl_verify() == expected_result
    else:
        with pytest.raises(expected_result):
            globus_sdk.config.get_ssl_verify()


@pytest.mark.parametrize(
    "value",
    [
        "invalid",
        1.0,
        object(),
        ca_bundle_directory,
        str(ca_bundle_directory),
        ca_bundle_non_existent,
        str(ca_bundle_non_existent),
    ],
)
def test_get_ssl_verify_rejects_bad_explicit_value(value, monkeypatch):
    monkeypatch.delenv("GLOBUS_SDK_VERIFY_SSL", raising=False)
    with pytest.raises(ValueError):
        globus_sdk.config.get_ssl_verify(value)


def test_get_ssl_verify_with_explicit_value():
    with mock.patch.dict(os.environ):
        os.environ["GLOBUS_SDK_VERIFY_SSL"] = "false"
        assert globus_sdk.config.get_ssl_verify(True) is True
        assert globus_sdk.config.get_ssl_verify(False) is False
        assert globus_sdk.config.get_ssl_verify(ca_bundle_file) == str(ca_bundle_file)
        assert globus_sdk.config.get_ssl_verify(str(ca_bundle_file)) == str(
            ca_bundle_file
        )
        os.environ["GLOBUS_SDK_VERIFY_SSL"] = "on"
        assert globus_sdk.config.get_ssl_verify(True) is True
        assert globus_sdk.config.get_ssl_verify(False) is False
        assert globus_sdk.config.get_ssl_verify(str(ca_bundle_file)) == str(
            ca_bundle_file
        )


@pytest.mark.parametrize(
    "value, expected_result",
    [(x, 1.0) for x in ["1.0", "1", " 1", "1.0  "]]
    + [(x, 0.0) for x in ["0", "0.0"]]
    + [("", 60.0)]
    + [("-1", None)]  # type: ignore
    + [(x, ValueError) for x in ["invalid", "no", "1.1.", "t", "f"]],  # type: ignore
)
def test_get_http_timeout(value, expected_result):
    """
    Confirms bool cast returns correct bools from sets of string values
    """
    with mock.patch.dict(os.environ):
        os.environ["GLOBUS_SDK_HTTP_TIMEOUT"] = value
        if expected_result is None or isinstance(expected_result, float):
            assert globus_sdk.config.get_http_timeout() == expected_result
        else:
            with pytest.raises(expected_result):
                globus_sdk.config.get_http_timeout()


def test_get_http_timeout_with_explicit_value():
    with mock.patch.dict(os.environ):
        os.environ["GLOBUS_SDK_HTTP_TIMEOUT"] = "120"
        assert globus_sdk.config.get_http_timeout(10) == 10.0
        assert globus_sdk.config.get_http_timeout(0) == 0.0
        del os.environ["GLOBUS_SDK_HTTP_TIMEOUT"]
        assert globus_sdk.config.get_http_timeout(60) == 60.0


def test_get_environment_name():
    with mock.patch.dict(os.environ):
        # set an environment value, ensure that it's returned
        os.environ["GLOBUS_SDK_ENVIRONMENT"] = "beta"
        assert globus_sdk.config.get_environment_name() == "beta"

        # clear that value, "production" should be returned
        del os.environ["GLOBUS_SDK_ENVIRONMENT"]
        assert globus_sdk.config.get_environment_name() == "production"

        # ensure that passing a value returns that value
        assert globus_sdk.config.get_environment_name("beta") == "beta"


def test_env_config_registration():
    with mock.patch.dict(globus_sdk.config.EnvConfig._registry):
        # should be None, we don't have an environment named 'moon'
        assert globus_sdk.config.EnvConfig.get_by_name("moon") is None

        # now, create the moon
        class MoonEnvConfig(globus_sdk.config.EnvConfig):
            envname = "moon"
            domain = "apollo.globus.org"

        # a lookup by "moon" should now get this config object
        assert globus_sdk.config.EnvConfig.get_by_name("moon") is MoonEnvConfig


def test_service_url_overrides():
    with mock.patch.dict(globus_sdk.config.EnvConfig._registry):

        class MarsEnvConfig(globus_sdk.config.EnvConfig):
            envname = "mars"
            domain = "mars.globus.org"
            auth_url = "https://perseverance.mars.globus.org/"

        # this one was customized
        assert (
            MarsEnvConfig.get_service_url("auth")
            == "https://perseverance.mars.globus.org/"
        )
        # but this one was not
        assert (
            MarsEnvConfig.get_service_url("search")
            == "https://search.api.mars.globus.org/"
        )


def test_service_url_from_env_var():
    with mock.patch.dict(os.environ):
        os.environ["GLOBUS_SDK_SERVICE_URL_TRANSFER"] = "https://transfer.example.org/"
        # environment setting gets ignored at this point -- only the override applies
        assert (
            globus_sdk.config.get_service_url("transfer", environment="preview")
            == "https://transfer.example.org/"
        )
        assert (
            globus_sdk.config.get_service_url("transfer", environment="production")
            == "https://transfer.example.org/"
        )
        # also try with a made up service
        os.environ["GLOBUS_SDK_SERVICE_URL_ION_CANNON"] = (
            "https://ion-cannon.example.org/"
        )
        assert (
            globus_sdk.config.get_service_url("ion_cannon", environment="production")
            == "https://ion-cannon.example.org/"
        )

        for env in ["sandbox", "test", "integration"]:
            os.environ["GLOBUS_SDK_ENVIRONMENT"] = env
            assert (
                globus_sdk.config.get_service_url("auth")
                == f"https://auth.{env}.globuscs.info/"
            )
            assert (
                globus_sdk.config.get_webapp_url()
                == f"https://app.{env}.globuscs.info/"
            )



import json

import pytest

from globus_sdk import __version__ as sdkversion
from globus_sdk.token_storage.legacy import SimpleJSONFileAdapter


def test_simplejson_reading_bad_data(tmp_path):
    # non-dict data at root
    foo_file = tmp_path / "foo.json"
    foo_file.write_text('["foobar"]')
    foo_adapter = SimpleJSONFileAdapter(str(foo_file))

    with pytest.raises(ValueError, match="Found non-dict root data while loading"):
        foo_adapter.get_by_resource_server()

    # non-dict data in 'by_rs'

    bar_file = tmp_path / "bar.json"
    bar_file.write_text(
        json.dumps(
            {"by_rs": [], "format_version": "1.0", "globus-sdk.version": sdkversion}
        )
    )
    bar_adapter = SimpleJSONFileAdapter(str(bar_file))

    with pytest.raises(ValueError, match="existing data file is malformed"):
        bar_adapter.get_by_resource_server()


def test_simplejson_reading_unsupported_format_version(tmp_path):
    # data appears valid, but lists a value for "format_version" which instructs the
    # adapter explicitly that it is in a format which is unknown / not supported
    foo_file = tmp_path / "foo.json"
    foo_file.write_text(
        json.dumps(
            {"by_rs": {}, "format_version": "0.0", "globus-sdk.version": sdkversion}
        )
    )
    adapter = SimpleJSONFileAdapter(str(foo_file))

    with pytest.raises(ValueError, match="existing data file is in an unknown format"):
        adapter.get_by_resource_server()
